@@ -1,16 +1,16 @@
-import glob
+import collections.abc
 from importlib.machinery import EXTENSION_SUFFIXES
 import os
 from pathlib import Path
 import sys
 import sysconfig
-from typing import Any, Optional, Tuple
+from typing import List, Optional, Tuple
 
 from .common import code_object_replace
 from .finder import ModuleFinder
 from .module import Module
 
-MINGW = sysconfig.get_platform() == "mingw"
+MINGW = sysconfig.get_platform().startswith("mingw")
 WIN32 = sys.platform == "win32"
 
 
@@ -20,8 +20,6 @@ def initialize(finder: ModuleFinder) -> None:
     automatic exclusions for various platforms.
     """
     # py2 modules that have been removed or renamed in py3
-    import collections.abc
-
     for name in collections.abc.__all__:
         finder.ExcludeModule("collections." + name)
     for name in (
@@ -105,6 +103,8 @@ def initialize(finder: ModuleFinder) -> None:
     # macos specfic removed in py3
     # https://docs.python.org/2.7/library/mac.html?highlight=removed
     finder.ExcludeModule("autoGIL")
+    finder.ExcludeModule("Carbon")
+    finder.ExcludeModule("ColorPicker")
     finder.ExcludeModule("EasyDialogs")
     finder.ExcludeModule("findertools")
     finder.ExcludeModule("FrameWork")
@@ -193,8 +193,14 @@ def initialize(finder: ModuleFinder) -> None:
         finder.ExcludeModule("riscosenviron")
         finder.ExcludeModule("riscospath")
         finder.ExcludeModule("rourl2path")
+    if not sys.platform.startswith("java"):
+        finder.ExcludeModule("com.sun")
+        finder.ExcludeModule("java")
+        finder.ExcludeModule("org.python")
     if not sys.platform.startswith("OpenVMS"):
         finder.ExcludeModule("vms_lib")
+    if "__pypy__" not in sys.builtin_module_names:
+        finder.ExcludeModule("__pypy__")
 
 
 def load_aiofiles(finder: ModuleFinder, module: Module) -> None:
@@ -250,9 +256,13 @@ def load_certifi(finder: ModuleFinder, module: Module) -> None:
         if sys.version_info < (3, 7):
             module.in_file_system = True
             return
-        cacert = __import__("certifi").where()
-        target = "certifi/" + os.path.basename(cacert)
-        finder.ZipIncludeFiles(cacert, target)
+        cacert = Path(__import__("certifi").where())
+        finder.ZipIncludeFiles(cacert, Path("certifi", cacert.name))
+
+
+def load__cffi_backend(finder: ModuleFinder, module: Module) -> None:
+    """Add the cffi metadata for _cffi_backend module."""
+    module.update_distribution("cffi")
 
 
 def load_cffi_cparser(finder: ModuleFinder, module: Module) -> None:
@@ -266,7 +276,7 @@ def load_cffi_cparser(finder: ModuleFinder, module: Module) -> None:
 
 
 def load_crc32c(finder: ModuleFinder, module: Module) -> None:
-    """The google.crc32c module requires _cffi_backend module"""
+    """The google.crc32c module requires _cffi_backend module."""
     finder.IncludeModule("_cffi_backend")
 
 
@@ -275,27 +285,21 @@ def load_clr(finder: ModuleFinder, module: Module) -> None:
     The pythonnet package (imported as 'clr') needs Python.Runtime.dll
     in runtime.
     """
-    module_dir = os.path.dirname(module.file)
     dll_name = "Python.Runtime.dll"
-    finder.IncludeFiles(
-        os.path.join(module_dir, dll_name), os.path.join("lib", dll_name)
-    )
+    finder.IncludeFiles(module.file.parent / dll_name, Path("lib", dll_name))
 
 
 def load_cryptography_hazmat_bindings__openssl(
     finder: ModuleFinder, module: Module
 ) -> None:
-    """The cryptography module requires the cffi module"""
-    finder.IncludeModule("cffi")
+    """The cryptography module requires the _cffi_backend module."""
+    finder.IncludeModule("_cffi_backend")
 
 
 def load_cryptography_hazmat_bindings__padding(
     finder: ModuleFinder, module: Module
 ) -> None:
-    """
-    The cryptography module requires the _cffi_backend module
-    (loaded implicitly).
-    """
+    """The cryptography module requires the _cffi_backend module."""
     finder.IncludeModule("_cffi_backend")
 
 
@@ -352,12 +356,12 @@ def pycryptodome_filename(dir_comps, filename):
     return os.path.join(root_lib, ".".join(dir_comps))
 """
     if not module.in_file_system and module.code is not None:
-        new_code = compile(PYCRYPTODOME_CODE_STR, module.file, "exec")
+        new_code = compile(PYCRYPTODOME_CODE_STR, str(module.file), "exec")
         co_func = new_code.co_consts[2]
         name = co_func.co_name
         code = module.code
         consts = list(code.co_consts)
-        for i in range(len(consts)):
+        for i in enumerate(consts):
             if isinstance(consts[i], type(code)) and consts[i].co_name == name:
                 consts[i] = co_func
                 break
@@ -371,8 +375,8 @@ def load__ctypes(finder: ModuleFinder, module: Module) -> None:
     """
     if WIN32 and sys.version_info >= (3, 8) and not MINGW:
         dll_name = "libffi-7.dll"
-        dll_path = os.path.join(sys.base_prefix, "DLLs", dll_name)
-        finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
+        dll_path = Path(sys.base_prefix, "DLLs", dll_name)
+        finder.IncludeFiles(dll_path, Path("lib", dll_name))
 
 
 def load_cx_Oracle(finder: ModuleFinder, module: Module) -> None:
@@ -503,6 +507,20 @@ def load_glib(finder: ModuleFinder, module: Module) -> None:
     )
 
 
+def load_googleapiclient(finder: ModuleFinder, module: Module) -> None:
+    """Add the googleapiclient metadata for googleapiclient package."""
+    module.update_distribution("google_api_python_client")
+
+
+def load_googleapiclient_discovery(
+    finder: ModuleFinder, module: Module
+) -> None:
+    """The googleapiclient.discovery module needs discovery_cache subpackage
+    in file system."""
+    discovery_cache = finder.IncludePackage("googleapiclient.discovery_cache")
+    discovery_cache.in_file_system = True
+
+
 def load_google_cloud_storage(finder: ModuleFinder, module: Module) -> None:
     """The google.cloud.storage package always uses the parent module."""
     finder.IncludePackage("google.cloud")
@@ -567,10 +585,10 @@ def load_llvmlite(finder: ModuleFinder, module: Module) -> None:
 
 def load_matplotlib(finder: ModuleFinder, module: Module) -> None:
     """The matplotlib package requires mpl-data subdirectory."""
-    data_path = os.path.join(module.path[0], "mpl-data")
-    target_path = os.path.join("lib", module.name, "mpl-data")
+    data_path = module.path[0] / "mpl-data"
+    target_path = Path("lib", module.name, "mpl-data")
     # After matplotlib 3.4 mpl-data is guaranteed to be a subdirectory.
-    if not os.path.isdir(data_path):
+    if not data_path.is_dir():
         data_path = __import__("matplotlib").get_data_path()
         need_patch = True
     else:
@@ -583,62 +601,54 @@ def load_matplotlib(finder: ModuleFinder, module: Module) -> None:
         return
     CODE_STR = f"""
 def _get_data_path():
-    return os.path.join(os.path.dirname(sys.executable), "{target_path}")
+    return os.path.join(os.path.dirname(sys.executable), "{target_path!s}")
 """
     for code_str in [CODE_STR, CODE_STR.replace("_get_data_", "get_data_")]:
-        new_code = compile(code_str, module.file, "exec")
+        new_code = compile(code_str, str(module.file), "exec")
         co_func = new_code.co_consts[0]
         name = co_func.co_name
         code = module.code
         consts = list(code.co_consts)
-        for i in range(len(consts)):
+        for i in enumerate(consts):
             if isinstance(consts[i], type(code)) and consts[i].co_name == name:
                 consts[i] = co_func
                 break
         module.code = code_object_replace(code, co_consts=consts)
 
 
-def load_mkl(finder: ModuleFinder, module: Module) -> None:
-    """The mkl package in conda."""
-    libs_dir = Path(sys.base_prefix) / "Library" / "bin"
-    if libs_dir.is_dir():
-        dest_dir = os.path.join("lib", "mkl")
-        for path in libs_dir.glob("mkl_*.dll"):
-            finder.IncludeFiles(path, os.path.join(dest_dir, path.name))
-        for path in libs_dir.glob("lib*.dll"):
-            finder.IncludeFiles(path, os.path.join(dest_dir, path.name))
-
-
 def load_numpy(finder: ModuleFinder, module: Module) -> None:
-    """"The numpy must be loaded as a package; support for pypi version and
-    numpy+mkl version (tested with 1.19.5+mkl and 1.20.3+mkl."""
+    """The numpy must be loaded as a package; support for pypi version and
+    numpy+mkl version - tested with 1.19.5+mkl, 1.20.3+mkl, 1.21.0+mkl,
+    1.21.1+mkl, 1.21.2+mkl and 1.21.2 from conda-forge."""
     finder.IncludePackage("numpy")
 
     if WIN32:
-        numpy_dir = Path(module.path[0])
-
-        # include mkl files
+        numpy_dir = module.path[0]
+        # numpy+mkl from: https://www.lfd.uci.edu/~gohlke/pythonlibs/#numpy
         libs_dir = numpy_dir / "DLLs"
+        if not libs_dir.is_dir():
+            # numpy+mkl from conda-forge
+            libs_dir = Path(sys.base_prefix, "Library", "bin")
         if libs_dir.is_dir():
-            dest_dir = os.path.join("lib", "numpy_mkl")
+            dest_dir = Path("lib", "numpy_mkl")
             for path in libs_dir.glob("mkl_*.dll"):
-                finder.IncludeFiles(path, os.path.join(dest_dir, path.name))
+                finder.IncludeFiles(path, dest_dir / path.name)
             for path in libs_dir.glob("lib*.dll"):
-                finder.IncludeFiles(path, os.path.join(dest_dir, path.name))
-            finder.AddConstant("MKL_PATH", dest_dir)
+                finder.IncludeFiles(path, dest_dir / path.name)
+            finder.AddConstant("MKL_PATH", str(dest_dir))
             finder.ExcludeModule("numpy.DLLs")
 
             # do not check dependencies already handled
             extension = EXTENSION_SUFFIXES[0]
             for path in numpy_dir.rglob(f"*{extension}"):
-                finder.ExcludeDependentFiles(str(path))
+                finder.ExcludeDependentFiles(path)
 
         # support for old versions (numpy <= 1.18.2)
         if not module.in_file_system:
             # copy any file at site-packages/numpy/.libs
             libs_dir = numpy_dir / ".libs"
             if libs_dir.is_dir():
-                finder.IncludeFiles(str(libs_dir), "lib")
+                finder.IncludeFiles(libs_dir, "lib")
 
     # exclude the tests
     finder.ExcludeModule("numpy.compat.tests")
@@ -779,8 +789,8 @@ def load_postgresql_lib(finder: ModuleFinder, module: Module) -> None:
     The postgresql.lib module requires the libsys.sql file to be included
     so make sure that file is included.
     """
-    filename = os.path.join(module.path[0], "libsys.sql")
-    finder.IncludeFiles(filename, os.path.basename(filename))
+    filename = "libsys.sql"
+    finder.IncludeFiles(module.path[0] / filename, filename)
 
 
 def load_pty(finder: ModuleFinder, module: Module) -> None:
@@ -843,76 +853,118 @@ def load_pyodbc(finder: ModuleFinder, module: Module) -> None:
         finder.IncludeModule(mod)
 
 
-# cache the QtCore module
-_qtcore = None
+# cache the QtCore library paths
+_qtcore_library_paths = []
 
 
-def _qt_implementation(module: Module) -> Tuple[str, Any]:
-    """Helper function to get name (PyQt5) and the QtCore module."""
-    global _qtcore
-    name = module.name.split(".")[0]
-    if _qtcore is None:
-        try:
-            _qtcore = __import__(name, fromlist=["QtCore"]).QtCore
-        except RuntimeError:
-            print(
-                "WARNING: Tried to load multiple incompatible Qt wrappers. "
-                "Some incorrect files may be copied."
-            )
-    return name, _qtcore
+def _qt_implementation(module: Module) -> str:
+    """Helper function to get the name of the Qt implementation (PyQt5)."""
+    return module.name.split(".")[0]
 
 
-def copy_qt_plugins(plugins, finder, qtcore):
-    """Helper function to find and copy Qt plugins."""
-
-    # Qt Plugins can either be in a plugins directory next to the Qt libraries,
-    # or in other locations listed by QCoreApplication.libraryPaths()
-    dir0 = os.path.join(os.path.dirname(qtcore.__file__), "plugins")
-    for libpath in qtcore.QCoreApplication.libraryPaths() + [dir0]:
-        sourcepath = os.path.join(str(libpath), plugins)
-        if os.path.exists(sourcepath):
-            finder.IncludeFiles(sourcepath, plugins)
-
-
-def sip_module_name(qtcore) -> str:
-    """
-    Returns the name of the sip module to import.
-    (As of 5.11, the distributed wheels no longer provided for the sip module
-    outside of the PyQt5 namespace).
-    """
-    version_string = qtcore.PYQT_VERSION_STR
+def _qt_library_paths(name: str) -> List[str]:
+    global _qtcore_library_paths
+    if _qtcore_library_paths:
+        return _qtcore_library_paths
     try:
-        pyqt_version_ints = tuple(int(c) for c in version_string.split("."))
-        if pyqt_version_ints >= (5, 11):
-            return "PyQt5.sip"
-    except Exception:
-        pass
-    return "sip"
+        qtcore = __import__(name, fromlist=["QtCore"]).QtCore
+    except RuntimeError:
+        print("WARNING: Tried to load multiple incompatible Qt ", end="")
+        print("wrappers. Some incorrect files may be copied.")
+        qtcore = None
+    else:
+        _qtcore_library_paths = [
+            Path(p) for p in qtcore.QCoreApplication.libraryPaths()
+        ]
+    if not _qtcore_library_paths:
+        # check the common location for conda
+        plugins_path = Path(sys.base_prefix, "Library", "plugins")
+        if plugins_path.exists():
+            _qtcore_library_paths.append(plugins_path)
+        elif qtcore:
+            # use a hack
+            app = qtcore.QCoreApplication([])
+            _qtcore_library_paths = [Path(p) for p in app.libraryPaths()]
+    if not _qtcore_library_paths and qtcore:
+        # Qt Plugins can be in a plugins directory next to the Qt libraries
+        pyqt5_root_dir = Path(qtcore.__file__).parent
+        _qtcore_library_paths.append(pyqt5_root_dir / "plugins")
+        _qtcore_library_paths.append(pyqt5_root_dir / "Qt5" / "plugins")
+        _qtcore_library_paths.append(pyqt5_root_dir / "Qt" / "plugins")
+    return _qtcore_library_paths
+
+
+def get_qt_plugins_paths(name: str, plugins: str) -> List[Tuple[str, str]]:
+    """Helper function to get a list of source and target paths of Qt plugins,
+    indicated to be used in include_files."""
+    include_files = []
+    for library_dir in _qt_library_paths(name):
+        if library_dir.parts[-1] != "plugins":
+            continue
+        source_path = library_dir / plugins
+        if not source_path.exists():
+            continue
+        if source_path.parts[-4] == name:
+            target_path = Path("lib").joinpath(*source_path.parts[-4:])
+        else:
+            # fallback plugins path to be used by load_PyQt5.
+            target_path = Path("lib") / name / "Qt" / "plugins" / plugins
+        include_files.append((source_path, target_path))
+    return include_files
+
+
+def copy_qt_plugins(name: str, plugins: str, finder: ModuleFinder) -> None:
+    """Helper function to find and copy Qt plugins."""
+    for source_path, target_path in get_qt_plugins_paths(name, plugins):
+        finder.IncludeFiles(source_path, target_path)
+
+
+def load_PyQt5(finder: ModuleFinder, module: Module) -> None:
+    """Inject code in PyQt5/PySide2 init to locate and load plugins."""
+    if module.code is None:
+        return
+    # With PyQt5 5.15.4, if the folder name contains non-ascii characters, the
+    # libraryPaths returns empty. Prior to this version, this doesn't happen.
+    # However, this hack will be used to workaround issues with anaconda and/or
+    # with the use of zip_include_packages too.
+    # With PySide2, the opposite happens. In PySide2 5.15.2, folders with non-
+    # ascii work, but in previous versions (5.15.1, 5.13.x, 5.12.0) they don't.
+    name = _qt_implementation(module)
+    code_string = module.file.read_text()
+    code_string += f"""
+# cx_Freeze patch start
+import os, sys
+from .QtCore import QCoreApplication
+
+executable_dir = os.path.dirname(sys.executable)
+pyqt5_root_dir = os.path.join(executable_dir, "lib", "{name}")
+plugins_dir = os.path.join(pyqt5_root_dir, "Qt5", "plugins")  # PyQt5 5.15.4
+if not os.path.isdir(plugins_dir):
+    plugins_dir = os.path.join(pyqt5_root_dir, "Qt", "plugins")  # others
+library_paths = [os.path.normcase(p) for p in QCoreApplication.libraryPaths()]
+if os.path.normcase(plugins_dir) not in library_paths:
+    library_paths = QCoreApplication.libraryPaths() + [plugins_dir]
+    QCoreApplication.setLibraryPaths(library_paths)
+# cx_Freeze patch end
+"""
+    module.code = compile(code_string, str(module.file), "exec")
 
 
 def load_PyQt5_phonon(finder: ModuleFinder, module: Module) -> None:
-    """
-    In Windows, phonon5.dll requires an additional dll phonon_ds94.dll to
-    be present in the build directory inside a folder phonon_backend.
-    """
-    if module.in_file_system:
-        return
-    _, qtcore = _qt_implementation(module)
+    """In Windows, phonon5.dll requires an additional dll phonon_ds94.dll to
+    be present in the build directory inside a folder phonon_backend."""
     if WIN32:
-        copy_qt_plugins("phonon_backend", finder, qtcore)
+        name = _qt_implementation(module)
+        copy_qt_plugins(name, "phonon_backend", finder)
 
 
 def load_PyQt5_Qt(finder: ModuleFinder, module: Module) -> None:
-    """
-    The PyQt5.Qt module is an extension module which imports a number of
+    """The PyQt5.Qt module is an extension module which imports a number of
     other modules and injects their namespace into its own. It seems a
     foolish way of doing things but perhaps there is some hidden advantage
     to this technique over pure Python; ignore the absence of some of
-    the modules since not every installation includes all of them.
-    """
-    if module.in_file_system:
-        return
-    name, _ = _qt_implementation(module)
+    the modules since not every installation includes all of them."""
+    name = _qt_implementation(module)
     finder.IncludeModule(f"{name}.QtCore")
     finder.IncludeModule(f"{name}.QtGui")
     for mod in (
@@ -935,81 +987,78 @@ def load_PyQt5_Qt(finder: ModuleFinder, module: Module) -> None:
 
 
 def load_PyQt5_QtCore(finder: ModuleFinder, module: Module) -> None:
-    """
-    The PyQt5.QtCore module implicitly imports the sip module and,
-    depending on configuration, the PyQt5._qt module.
-    """
-    if module.in_file_system:
-        return
-    name, qtcore = _qt_implementation(module)
-    finder.IncludeModule(sip_module_name(qtcore))
+    """The PyQt5.QtCore module implicitly imports the sip module and,
+    depending on configuration, the PyQt5._qt module."""
+    name = _qt_implementation(module)
+    try:
+        finder.IncludeModule(f"{name}.sip")  # PyQt5 >= 5.11
+    except ImportError:
+        finder.IncludeModule("sip")
     try:
         finder.IncludeModule(f"{name}._qt")
     except ImportError:
         pass
 
 
-def load_PyQt5_uic(finder: ModuleFinder, module: Module) -> None:
-    """
-    The uic module makes use of "plugins" that need to be read directly and
-    cannot be frozen; the PyQt5.QtWebKit and PyQt5.QtNetwork modules are
-    also implicity loaded.
-    """
-    if module.in_file_system:
-        return
-    name, _ = _qt_implementation(module)
-    source_dir = os.path.join(module.path[0], "widget-plugins")
-    finder.IncludeFiles(source_dir, f"{name}.uic.widget-plugins")
-    finder.IncludeModule(f"{name}.QtNetwork")
-    try:
-        finder.IncludeModule(f"{name}.QtWebKit")
-    except ImportError:
-        pass
-
-
 def load_PyQt5_QtGui(finder: ModuleFinder, module: Module) -> None:
-    """
-    There is a chance that GUI will use some image formats
-    add the image format plugins.
-    """
-    if module.in_file_system:
-        return
-    name, qtcore = _qt_implementation(module)
+    """There is a chance that GUI will use some image formats
+    add the image format plugins."""
+    name = _qt_implementation(module)
     finder.IncludeModule(f"{name}.QtCore")
-    copy_qt_plugins("imageformats", finder, qtcore)
+    copy_qt_plugins(name, "imageformats", finder)
     # On Qt5, we need the platform plugins. For simplicity, we just copy
     # any that are installed.
-    copy_qt_plugins("platforms", finder, qtcore)
+    copy_qt_plugins(name, "platforms", finder)
 
 
 def load_PyQt5_QtMultimedia(finder: ModuleFinder, module: Module) -> None:
-    if module.in_file_system:
-        return
-    name, qtcore = _qt_implementation(module)
+    name = _qt_implementation(module)
     finder.IncludeModule(f"{name}.QtCore")
     finder.IncludeModule(f"{name}.QtMultimediaWidgets")
-    copy_qt_plugins("mediaservice", finder, qtcore)
+    copy_qt_plugins(name, "mediaservice", finder)
 
 
 def load_PyQt5_QtPrintSupport(finder: ModuleFinder, module: Module) -> None:
-    if module.in_file_system:
-        return
-    _, qtcore = _qt_implementation(module)
-    copy_qt_plugins("printsupport", finder, qtcore)
+    name = _qt_implementation(module)
+    copy_qt_plugins(name, "printsupport", finder)
 
 
 def load_PyQt5_QtWebKit(finder: ModuleFinder, module: Module) -> None:
-    if module.in_file_system:
-        return
-    name, _ = _qt_implementation(module)
+    name = _qt_implementation(module)
     finder.IncludeModule(f"{name}.QtNetwork")
     finder.IncludeModule(f"{name}.QtGui")
 
 
 def load_PyQt5_QtWidgets(finder: ModuleFinder, module: Module) -> None:
-    if module.in_file_system:
-        return
-    finder.IncludeModule("PyQt5.QtGui")
+    name = _qt_implementation(module)
+    finder.IncludeModule(f"{name}.QtGui")
+
+
+def load_PyQt5_uic(finder: ModuleFinder, module: Module) -> None:
+    """The uic module makes use of "plugins" that need to be read directly and
+    cannot be frozen; the PyQt5.QtWebKit and PyQt5.QtNetwork modules are
+    also implicity loaded."""
+    name = _qt_implementation(module)
+    finder.IncludeModule(f"{name}.QtNetwork")
+    try:
+        finder.IncludeModule(f"{name}.QtWebKit")
+    except ImportError:
+        pass
+    source_dir = module.path[0] / "widget-plugins"
+    finder.IncludeFiles(source_dir, f"{name}.uic.widget-plugins")
+
+
+# PySide2 start
+load_PySide2 = load_PyQt5
+load_PySide2_Qt = load_PyQt5_Qt
+# load_PySide2_QtCore is not necessary
+load_PySide2_QtGui = load_PyQt5_QtGui
+load_PySide2_QtMultimedia = load_PyQt5_QtMultimedia
+load_PySide2_QtPrintSupport = load_PyQt5_QtPrintSupport
+load_PySide2_QtWebKit = load_PyQt5_QtWebKit
+load_PySide2_QtWidgets = load_PyQt5_QtWidgets
+load_PySide2_uic = load_PyQt5_uic
+# PySide2 end
 
 
 def load_pyqtgraph(finder: ModuleFinder, module: Module) -> None:
@@ -1036,10 +1085,9 @@ def load_pythoncom(finder: ModuleFinder, module: Module) -> None:
     the target directory.
     """
     pythoncom = __import__("pythoncom")
+    filename = Path(pythoncom.__file__)
     finder.IncludeFiles(
-        pythoncom.__file__,
-        os.path.join("lib", os.path.basename(pythoncom.__file__)),
-        copy_dependent_files=False,
+        filename, Path("lib", filename.name), copy_dependent_files=False
     )
 
 
@@ -1048,28 +1096,26 @@ def load_pytz(finder: ModuleFinder, module: Module) -> None:
     The pytz module requires timezone data to be found in a known directory
     or in the zip file where the package is written.
     """
-    target_path = os.path.join("lib", "pytz", "zoneinfo")
-    data_path = os.path.join(module.path[0], "zoneinfo")
-    if not os.path.isdir(data_path):
+    target_path = Path("lib", "pytz", "zoneinfo")
+    data_path = module.path[0] / "zoneinfo"
+    if not data_path.is_dir():
         # Fedora (and possibly other systems) use a separate location to
         # store timezone data so look for that here as well
         pytz = __import__("pytz")
-        data_path = (
+        data_path = Path(
             getattr(pytz, "_tzinfo_dir", None)
             or os.getenv("PYTZ_TZDATADIR")
             or "/usr/share/zoneinfo"
         )
-        if data_path.endswith(os.sep):
-            data_path = data_path[:-1]
-        if os.path.isdir(data_path):
-            finder.AddConstant("PYTZ_TZDATADIR", target_path)
-    if os.path.isdir(data_path):
+        if data_path.is_dir():
+            finder.AddConstant("PYTZ_TZDATADIR", str(target_path))
+    if data_path.is_dir():
         if module.in_file_system:
             finder.IncludeFiles(
                 data_path, target_path, copy_dependent_files=False
             )
         else:
-            finder.ZipIncludeFiles(data_path, "pytz/zoneinfo")
+            finder.ZipIncludeFiles(data_path, Path("pytz", "zoneinfo"))
 
 
 def load_pywintypes(finder: ModuleFinder, module: Module) -> None:
@@ -1081,10 +1127,9 @@ def load_pywintypes(finder: ModuleFinder, module: Module) -> None:
     target directory.
     """
     pywintypes = __import__("pywintypes")
+    filename = Path(pywintypes.__file__)
     finder.IncludeFiles(
-        pywintypes.__file__,
-        os.path.join("lib", os.path.basename(pywintypes.__file__)),
-        copy_dependent_files=False,
+        filename, Path("lib", filename.name), copy_dependent_files=False
     )
 
 
@@ -1231,12 +1276,10 @@ def load_sqlite3(finder: ModuleFinder, module: Module) -> None:
     """
     if WIN32 and not MINGW:
         dll_name = "sqlite3.dll"
-        dll_path = os.path.join(sys.base_prefix, "DLLs", dll_name)
-        if not os.path.exists(dll_path):
-            dll_path = os.path.join(
-                sys.base_prefix, "Library", "bin", dll_name
-            )
-        finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
+        dll_path = Path(sys.base_prefix, "DLLs", dll_name)
+        if not dll_path.exists():
+            dll_path = Path(sys.base_prefix, "Library", "bin", dll_name)
+        finder.IncludeFiles(dll_path, Path("lib", dll_name))
     finder.IncludePackage("sqlite3")
 
 
@@ -1252,11 +1295,9 @@ def load_ssl(finder: ModuleFinder, module: Module) -> None:
     """
     if WIN32 and sys.version_info >= (3, 7) and not MINGW:
         for dll_search in ["libcrypto-*.dll", "libssl-*.dll"]:
-            for dll_path in glob.glob(
-                os.path.join(sys.base_prefix, "DLLs", dll_search)
-            ):
-                dll_name = os.path.basename(dll_path)
-                finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
+            libs_dir = Path(sys.base_prefix, "DLLs")
+            for dll_path in libs_dir.glob(dll_search):
+                finder.IncludeFiles(dll_path, Path("lib", dll_path.name))
 
 
 def load_sysconfig(finder: ModuleFinder, module: Module) -> None:
@@ -1301,16 +1342,16 @@ def load_tkinter(finder: ModuleFinder, module: Module) -> None:
                 lib_texts = os.environ[env_name]
             except KeyError:
                 if MINGW:
-                    lib_texts = os.path.join(sys.base_prefix, "lib", dir_name)
+                    lib_texts = Path(sys.base_prefix, "lib", dir_name)
                 else:
-                    lib_texts = os.path.join(sys.base_prefix, "tcl", dir_name)
-            target_path = os.path.join("lib", "tkinter", dir_name)
-            finder.AddConstant(env_name, target_path)
+                    lib_texts = Path(sys.base_prefix, "tcl", dir_name)
+            target_path = Path("lib", "tkinter", dir_name)
+            finder.AddConstant(env_name, str(target_path))
             finder.IncludeFiles(lib_texts, target_path)
             if not MINGW:
                 dll_name = dir_name.replace(".", "") + "t.dll"
-                dll_path = os.path.join(sys.base_prefix, "DLLs", dll_name)
-                finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
+                dll_path = Path(sys.base_prefix, "DLLs", dll_name)
+                finder.IncludeFiles(dll_path, Path("lib", dll_name))
 
 
 def load_twisted_conch_ssh_transport(
@@ -1352,8 +1393,7 @@ def load_win32com(finder: ModuleFinder, module: Module) -> None:
     the sibling directory called win32comext; simulate that by changing the
     search path in a similar fashion here.
     """
-    base_dir = os.path.dirname(os.path.dirname(module.file))
-    module.path.append(os.path.join(base_dir, "win32comext"))
+    module.path.append(module.file.parent.parent / "win32comext")
 
 
 def load_win32file(finder: ModuleFinder, module: Module) -> None:
@@ -1373,8 +1413,7 @@ def load_wx_lib_pubsub_core(finder: ModuleFinder, module: Module) -> None:
     that this only works if the import of wx.lib.pubsub.setupkwargs
     occurs first.
     """
-    dir_name = os.path.dirname(module.file)
-    module.path.insert(0, os.path.join(dir_name, "kwargs"))
+    module.path.insert(0, module.file.parent / "kwargs")
 
 
 def load_Xlib_display(finder: ModuleFinder, module: Module) -> None:
@@ -1395,7 +1434,7 @@ def load_Xlib_support_connect(finder: ModuleFinder, module: Module) -> None:
     The Xlib.support.connect module implicitly loads a platform specific
     module; make sure this happens.
     """
-    if sys.platform.split("-")[0] == "OpenVMS":
+    if sys.platform.split("-", maxsplit=1)[0] == "OpenVMS":
         module_name = "vms_connect"
     else:
         module_name = "unix_connect"
@@ -1420,52 +1459,56 @@ def load_xml_etree_cElementTree(finder: ModuleFinder, module: Module) -> None:
 
 
 def load_zmq(finder: ModuleFinder, module: Module) -> None:
-    """
-    The zmq package loads zmq.backend.cython dynamically and links
-    dynamically to zmq.libzmq.
-    """
+    """The zmq package loads zmq.backend.cython dynamically and links
+    dynamically to zmq.libzmq or shared lib. Tested in pyzmq 16.0.4 (py36),
+    19.0.2 (MSYS2 py39) up to 22.2.1 (from pip and from conda)."""
     finder.IncludePackage("zmq.backend.cython")
     if WIN32:
         # For pyzmq 22 the libzmq dependencies are located in
         # site-packages/pyzmq.libs
         libzmq_folder = "pyzmq.libs"
-        libs_dir = os.path.join(os.path.dirname(module.path[0]), libzmq_folder)
-        if os.path.exists(libs_dir):
-            finder.IncludeFiles(libs_dir, os.path.join("lib", libzmq_folder))
-            return
-        # Include the bundled libzmq library, if it exists
-        try:
-            libzmq = __import__("zmq", fromlist=["libzmq"]).libzmq
-            filename = os.path.basename(libzmq.__file__)
-            finder.IncludeFiles(
-                os.path.join(module.path[0], filename), filename
-            )
-        except (ImportError, AttributeError):
-            pass  # assume libzmq is not bundled
+        libs_dir = module.path[0].parent / libzmq_folder
+        if libs_dir.exists():
+            finder.IncludeFiles(libs_dir, Path("lib", libzmq_folder))
+    # Include the bundled libzmq library, if it exists
+    try:
+        finder.IncludeModule("zmq.libzmq")
+    except ImportError:
+        pass  # assume libzmq is not bundled
+    finder.ExcludeModule("zmq.tests")
 
 
 def load_zoneinfo(finder: ModuleFinder, module: Module) -> None:
-    """
-    The zoneinfo package requires timezone data, that
-    can be the in tzdata package, if installed.
-    """
-    tzdata: Optional[Module]
+    """The zoneinfo package requires timezone data, that
+    can be the in tzdata package, if installed."""
+    tzdata: Optional[Module] = None
+    source: Optional[Path] = None
     try:
         tzdata = finder.IncludePackage("tzdata")
+        # store tzdata along with zoneinfo
+        tzdata.in_file_system = module.in_file_system
     except ImportError:
-        tzdata = None
+        zoneinfo = __import__(module.name, fromlist=["TZPATH"])
+        if zoneinfo.TZPATH:
+            for path in zoneinfo.TZPATH:
+                if path.endswith("zoneinfo"):
+                    source = Path(path)
+                    break
+        if source:
+            # without tzdata, copy only zoneinfo directory
+            # in Linux: /usr/share/zoneinfo
+            target = Path("lib", "tzdata", "zoneinfo")
+            finder.IncludeFiles(source, target, copy_dependent_files=False)
+            finder.AddConstant("PYTHONTZPATH", str(source))
     if tzdata is None:
         return
-    # store tzdata along with zoneinfo
-    tzdata.in_file_system = module.in_file_system
+    # when the tzdata exists, copy other files in this directory
+    source = tzdata.path[0]
+    target = Path("lib", "tzdata")
     if tzdata.in_file_system:
-        finder.IncludeFiles(
-            tzdata.path[0],
-            os.path.join("lib", "tzdata"),
-            copy_dependent_files=False,
-        )
+        finder.IncludeFiles(source, target, copy_dependent_files=False)
     else:
-        finder.ZipIncludeFiles(tzdata.path[0], "tzdata")
+        finder.ZipIncludeFiles(source, "tzdata")
 
 
 load_backports_zoneinfo = load_zoneinfo
